<html>
  <!--
    Copyright 2011, Google Inc. All Rights Reserved.
    
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    
    http://www.apache.org/licenses/LICENSE-2.0
    
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
    
    author: pti@google.com (Prashant Tiwari)
  -->
  <head>
    <link type="text/css" rel="stylesheet" href="/css/index.css"></link>
    <script src="/js/index.js" type="text/javascript"></script>
  </head>
  <body onload="onload();" style="text-align: center">
    <div class="outer">
      <div id="header">
        <div id="titleholder">
          <h1 style="text-align: center">{{ app_title }}</h1>
        </div>
        <div id="titlecrumb">
          {% if oauth_token %}
          <div>
            <a href="/start_over?two_legged_oauth={{ two_legged_oauth }}">
            {{ start_over_text }}</a>
          </div>
          {% endif %}
        </div>
      <div style="clear: both;"></div>
    </div>