/*
Copyright (c) 2008, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.net/yui/license.txt
version: 3.0.0pr1
*/
/* base.css, part of YUI's CSS Foundation */
.yui-cssbase h1 {
	/*18px via YUI Fonts CSS foundation*/
	font-size:138.5%;  
}
.yui-cssbase h2 {
	/*16px via YUI Fonts CSS foundation*/
	font-size:123.1%; 
}
.yui-cssbase h3 {
	/*14px via YUI Fonts CSS foundation*/
	font-size:108%;  
}
.yui-cssbase h1,.yui-cssbase h2,.yui-cssbase h3 {
	/* top & bottom margin based on font size */
	margin:1em 0;
}
.yui-cssbase h1,.yui-cssbase h2,.yui-cssbase h3,.yui-cssbase h4,.yui-cssbase h5,.yui-cssbase h6,.yui-cssbase strong {
	/*bringing boldness back to headers and the strong element*/
	font-weight:bold; 
}
.yui-cssbase abbr,.yui-cssbase acronym {
	/*indicating to users that more info is available */
	border-bottom:1px dotted #000;
	cursor:help;
} 
.yui-cssbase em {
	/*bringing italics back to the em element*/
	font-style:italic;
}
.yui-cssbase blockquote,.yui-cssbase ul,.yui-cssbase ol,.yui-cssbase dl {
	/*giving blockquotes and lists room to breath*/
	margin:1em;
}
.yui-cssbase ol,.yui-cssbase ul,.yui-cssbase dl {
	/*bringing lists on to the page with breathing room */
	margin-left:2em;
}
.yui-cssbase ol li {
	/*giving OL's LIs generated numbers*/
	list-style: decimal outside;	
}
.yui-cssbase ul li {
	/*giving UL's LIs generated disc markers*/
	list-style: disc outside;
}
.yui-cssbase dl dd {
	/*giving UL's LIs generated numbers*/
	margin-left:1em;
}
.yui-cssbase th,.yui-cssbase td {
	/*borders and padding to make the table readable*/
	border:1px solid #000;
	padding:.5em;
}
.yui-cssbase th {
	/*distinguishing table headers from data cells*/
	font-weight:bold;
	text-align:center;
}
.yui-cssbase caption {
	/*coordinated margin to match cell's padding*/
	margin-bottom:.5em;
	/*centered so it doesn't blend in to other content*/
	text-align:center;
}
.yui-cssbase p,.yui-cssbase fieldset,.yui-cssbase table,.yui-cssbase pre {
	/*so things don't run into each other*/
	margin-bottom:1em;
}
/* setting a consistent width, 160px; 
   control of type=file still not possible */
.yui-cssbase input[type=text],.yui-cssbase input[type=password],.yui-cssbase textarea{width:12.25em;*width:11.9em;}
